/* -*- Mode: C++ -*- */

/*
    libscoach: Soccer Coach library for use with the SoccerServer system 

    Copyright (C) 2001  Patrick Riley, Paul Carpenter, Gal Kaminka, Manuela Veloso
    Copyright (C) 2001 RoboCup Soccer Server Maintainance Group.
    	Patrick Riley, Tom Howard, Itsuki Noda,	Mikhail Prokopenko, Jan Wendler 
    Copyright (C) 2000, 2001 RoboCup Soccer Server Maintainance Group.
    	Patrick Riley, Tom Howard, Daniel Polani, Itsuki Noda,
	Mikhail Prokopenko, Jan Wendler 

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/* This is a modified version of a file distributed with the SoccerServer system
   http://sserver.sourceforge.net/
   Patrick Riley (pfr@cs.cmu.edu) 08/2001
*/

#include <iostream.h>
#include <stdio.h>
#include "coach_lang.h"
#ifdef WITH_DRAW
#include "FieldImage.h"
#endif


#define FREEFORM_MESS_LENGTH 128

/******** Meta Message ********************/

void CM_MetaMessage::clearTokens()
{
  vector<CM_MetaToken*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter) 
    delete (*token_iter);
}

void CM_MetaMessage::Print(ostream& out)
{
  out << "(meta";
  vector<CM_MetaToken*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter) 
    out << " " << **token_iter;
  out << ")";
}

void CM_MetaMessage::PrintPretty(ostream& out, string line_header)
{
  out << line_header << "Meta" << endl;
  vector<CM_MetaToken*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter) 
    (*token_iter)->PrintPretty(out, line_header + " - ");
    //out << line_header << "  " << (**token_iter) << endl;
}

void CM_MetaMessage::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterMessage(this);
  vector<CM_MetaToken*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter)
    (*token_iter)->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterMessage(this);
}

CM_Message* CM_MetaMessage::clone()
{
  vector<CM_MetaToken*> newV;
  vector<CM_MetaToken*>::iterator iter;
  for (iter = vTokens.begin(); iter != vTokens.end(); ++iter) 
    newV.push_back((*iter)->clone());
  return new CM_MetaMessage(newV);
}


#ifdef WITH_DRAW
void CM_MetaMessage::draw(FieldImage* pfi, const char* fstem)
{
  ostrstream out;
  out << *this << ends;
  pfi->addLegendLine(out.str());
  out.freeze(0);
  char fn[MAX_FILE_LEN];
  sprintf(fn, "%s.gif", fstem);
  pfi->writeImage(fn);
}
#endif



/******** Freeform Message ********************/

CM_FreeformMessage::CM_FreeformMessage(char* str)
  : CM_Message()
{
  this->str = new char[FREEFORM_MESS_LENGTH+1];
  setStr(str);
}

CM_FreeformMessage:: ~CM_FreeformMessage()
{
  delete [] str;
}

void CM_FreeformMessage::setStr(char* s)
{
  if (s == NULL)
    s[0] = 0;
  else {
    strncpy(str,s,FREEFORM_MESS_LENGTH);
    str[FREEFORM_MESS_LENGTH] = 0;
  }
}

void CM_FreeformMessage::setStrQuoteDelim(char* s)
{
  strncpy(str,s+1,FREEFORM_MESS_LENGTH);
  str[FREEFORM_MESS_LENGTH] = 0;
  int len = strlen(str);
  if (str[len-1] == '"')
    str[len-1] = 0;
  else 
    printf("no end quote! %c", str[len-1]);
}


void CM_FreeformMessage::Print(ostream& out)
{
  out << "(freeform \"" << str << "\")";
}

void CM_FreeformMessage::PrintPretty(ostream& out, string line_header)
{
  out << line_header << "Freeform" << endl;
  out << line_header << "  " << "\"" << str << "\"" << endl;
}

#ifdef WITH_DRAW
void CM_FreeformMessage::draw(FieldImage* pfi, const char* fstem)
{
  ostrstream out;
  out << *this << ends;
  pfi->addLegendLine(out.str());
  out.freeze(0);
  char fn[MAX_FILE_LEN];
  sprintf(fn, "%s.gif", fstem);
  pfi->writeImage(fn);
}
#endif


/******** Info Message ********************/

void CM_InfoMessage::Print(ostream& out)
{
  out << "(info";
  vector<CM_Token*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter) 
    out << " " << **token_iter;
  out << ")";
}

void CM_InfoMessage::clearTokens()
{
  vector<CM_Token*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter) 
    delete (*token_iter);
  vTokens.clear();
}

void CM_InfoMessage::PrintPretty(ostream& out, string line_header)
{
  out << line_header << "Info" << endl;
  vector<CM_Token*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter) 
    (*token_iter)->PrintPretty(out, line_header + " - ");
    //out << line_header << "  " << (**token_iter) << endl;
}

void CM_InfoMessage::linkNamedElements(const CoachMessageRepository* pCMR) 
{
  vector<CM_Token*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter)
    (*token_iter)->linkNamedElements(pCMR);
}

void CM_InfoMessage::updateNamedCondition(const char* name, CM_Condition* p)
{
  vector<CM_Token*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter)
    (*token_iter)->updateNamedCondition(name, p);
}

void CM_InfoMessage::updateNamedDirective(const char* name, CM_Directive* p)
{
  vector<CM_Token*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter)
    (*token_iter)->updateNamedDirective(name, p);
}

void CM_InfoMessage::updateNamedRegion   (const char* name, RegionPtr p)
{
  vector<CM_Token*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter)
    (*token_iter)->updateNamedRegion(name, p);
}

void CM_InfoMessage::updateNamedAction   (const char* name, CM_Action* p)
{
  vector<CM_Token*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter)
    (*token_iter)->updateNamedAction(name, p);
}

void CM_InfoMessage::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterMessage(this);
  vector<CM_Token*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter)
    (*token_iter)->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterMessage(this);  
}

CM_Message* CM_InfoMessage::clone()
{
  vector<CM_Token*> newV;
  vector<CM_Token*>::iterator iter;
  for (iter = vTokens.begin(); iter != vTokens.end(); ++iter) 
    newV.push_back((*iter)->clone());
  return new CM_InfoMessage(newV);
}

#ifdef WITH_DRAW
void CM_InfoMessage::draw(FieldImage* pfi, const char* fstem)
{
  int idx = 0;
  vector<CM_Token*>::iterator iter;
  for (iter = vTokens.begin(); iter != vTokens.end(); ++iter, ++idx) {
    pfi->erase();
    pfi->addFieldLines();
    pfi->addLegendLine("INFO");
    (*iter)->draw(pfi);
    char fn[MAX_FILE_LEN];
    sprintf(fn, "%s%03d.gif", fstem, idx);
    pfi->writeImage(fn);
  }
}
#endif


/******** Advice Message ********************/
  
void CM_AdviceMessage::Print(ostream& out)
{
  out << "(advice";
  vector<CM_Token*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter) 
    out << " " << **token_iter;
  out << ")";
}

void CM_AdviceMessage::clearTokens()
{
  vector<CM_Token*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter) 
    delete (*token_iter);
  vTokens.clear();
}

void CM_AdviceMessage::PrintPretty(ostream& out, string line_header)
{
  out << line_header << "Advice" << endl;
  vector<CM_Token*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter) 
    (*token_iter)->PrintPretty(out, line_header + " - ");
    //out << line_header << " " << (**token_iter) << endl;
}

void CM_AdviceMessage::linkNamedElements(const CoachMessageRepository* pCMR) 
{
  vector<CM_Token*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter)
    (*token_iter)->linkNamedElements(pCMR);
}

void CM_AdviceMessage::updateNamedCondition(const char* name, CM_Condition* p)
{
  vector<CM_Token*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter)
    (*token_iter)->updateNamedCondition(name, p);
}

void CM_AdviceMessage::updateNamedDirective(const char* name, CM_Directive* p)
{
  vector<CM_Token*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter)
    (*token_iter)->updateNamedDirective(name, p);
}

void CM_AdviceMessage::updateNamedRegion   (const char* name, RegionPtr p)
{
  vector<CM_Token*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter)
    (*token_iter)->updateNamedRegion(name, p);
}

void CM_AdviceMessage::updateNamedAction   (const char* name, CM_Action* p)
{
  vector<CM_Token*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter)
    (*token_iter)->updateNamedAction(name, p);
}

void CM_AdviceMessage::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterMessage(this);
  vector<CM_Token*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter)
    (*token_iter)->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterMessage(this);  
}

CM_Message* CM_AdviceMessage::clone()
{
  vector<CM_Token*> newV;
  vector<CM_Token*>::iterator iter;
  for (iter = vTokens.begin(); iter != vTokens.end(); ++iter) 
    newV.push_back((*iter)->clone());
  return new CM_AdviceMessage(newV);
}

#ifdef WITH_DRAW
void CM_AdviceMessage::draw(FieldImage* pfi, const char* fstem)
{
  int idx = 0;
  vector<CM_Token*>::iterator iter;
  for (iter = vTokens.begin(); iter != vTokens.end(); ++iter, ++idx) {
    pfi->erase();
    pfi->addFieldLines();
    pfi->addLegendLine("ADVICE");
    (*iter)->draw(pfi);
    char fn[MAX_FILE_LEN];
    sprintf(fn, "%s%03d.gif", fstem, idx);
    pfi->writeImage(fn);
  }
}
#endif

/******** Define Message ********************/
  
void CM_DefineMessage::Print(ostream& out)
{
  out << "(define";
  vector<CM_DefineToken*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter) 
    out << " " << **token_iter;
  out << ")";
}

void CM_DefineMessage::clearTokens()
{
  vector<CM_DefineToken*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter) 
    delete (*token_iter);
  vTokens.clear();
}

void CM_DefineMessage::PrintPretty(ostream& out, string line_header)
{
  out << line_header << "Define" << endl;
  vector<CM_DefineToken*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter) 
    (*token_iter)->PrintPretty(out, line_header + " - ");
    //out << line_header << "  " << (**token_iter) << endl;
}

void CM_DefineMessage::linkNamedElements(const CoachMessageRepository* pCMR)
{
  vector<CM_DefineToken*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter) 
    (*token_iter)->linkNamedElements(pCMR);
}

void CM_DefineMessage::updateNamedCondition(const char* name, CM_Condition* p)
{
  vector<CM_DefineToken*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter) 
    (*token_iter)->updateNamedCondition(name, p);
}

void CM_DefineMessage::updateNamedDirective(const char* name, CM_Directive* p)
{
  vector<CM_DefineToken*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter) 
    (*token_iter)->updateNamedDirective(name, p);
}

void CM_DefineMessage::updateNamedRegion(const char* name, RegionPtr p)
{
  vector<CM_DefineToken*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter) 
    (*token_iter)->updateNamedRegion(name, p);
}

void CM_DefineMessage::updateNamedAction(const char* name, CM_Action* p)
{
  vector<CM_DefineToken*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter) 
    (*token_iter)->updateNamedAction(name, p);
}

void CM_DefineMessage::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterMessage(this);
  vector<CM_DefineToken*>::iterator token_iter;
  for (token_iter = vTokens.begin(); token_iter != vTokens.end(); ++token_iter)
    (*token_iter)->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterMessage(this);  
}

CM_Message* CM_DefineMessage::clone()
{
  vector<CM_DefineToken*> newV;
  vector<CM_DefineToken*>::iterator iter;
  for (iter = vTokens.begin(); iter != vTokens.end(); ++iter) 
    newV.push_back((*iter)->clone());
  return new CM_DefineMessage(newV);
}

#ifdef WITH_DRAW
void CM_DefineMessage::draw(FieldImage* pfi, const char* fstem)
{
  ostrstream out;
  out << *this << ends;
  pfi->addLegendLine(out.str());
  out.freeze(0);
  char fn[MAX_FILE_LEN];
  sprintf(fn, "%s.gif", fstem);
  pfi->writeImage(fn);
}
#endif

