/* -*- Mode: C++ -*- */

/*
    libscoach: Soccer Coach library for use with the SoccerServer system 

    Copyright (C) 2001  Patrick Riley, Paul Carpenter, Gal Kaminka, Manuela Veloso
    Copyright (C) 2001 RoboCup Soccer Server Maintainance Group.
    	Patrick Riley, Tom Howard, Itsuki Noda,	Mikhail Prokopenko, Jan Wendler 
    Copyright (C) 2000, 2001 RoboCup Soccer Server Maintainance Group.
    	Patrick Riley, Tom Howard, Daniel Polani, Itsuki Noda,
	Mikhail Prokopenko, Jan Wendler 

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/* This is a modified version of a file distributed with the SoccerServer system
   http://sserver.sourceforge.net/
   Patrick Riley (pfr@cs.cmu.edu) 08/2001
*/

#include "coach_lang_comp.h"

#include "coach_mess.h"

#ifdef WITH_DRAW
#include "FieldImage.h"
#endif
/******** type definitions **********/
char* ActBallTo_strings[] =
{ "NONE", "p", "d", "a" };
char* BallMoveToken_strings[] =
{ "NONE", "p", "d", "c", "s", "ALL" };
char* CM_PlayModeType_strings[] = {
  "",
  "bko",
  "time_over",
  "play_on",
  "ko_our",
  "ko_opp",
  "ki_our",
  "ki_opp",
  "fk_our",
  "fk_opp",
  "ck_our",
  "ck_opp",
  "gk_our",
  "gk_opp",
  "gc_our",
  "gc_opp",
  "ag_our",
  "ag_opp" };
  

/******** Misc Function **********************/
char* strip_quotes(char* str)
{
  int len = strlen(str);
  if (len == 0) return str;
  if (str[len-1] == '"')
    str[len-1] = 0;
  return (str[0] == '"' ? str+1 : str);
}


/******** Utility Classes ********************/
void UnumSet::addNum(int i)
{
  if (i < 0 || i > team_size)
    return;
  if (i==0) {
    entries = ~0;
  } else {
    entries |= (1 << (i-1));
  }
}

void UnumSet::removeNum(int i)
{
  if (i < 0 || i > team_size)
    return;
  if (i==0) {
    clear();
  } else {
    entries &= ~(1 << (i-1));
  }
}

bool UnumSet::isNumMember(int i) const
{
  if (i < 0 || i > team_size)
    return false;
  if (i==0) {
    return (~(((~0) << team_size) | entries)) == 0;
  } else {
    return !!(entries & (1 << (i-1)));
  }
}

void UnumSet::Print(ostream& out) const
{
  char buf[2]; //this is used to handle spacing
  buf[0] = 0;
  buf[1] = 0;
  out << "{";
  if (isNumMember(0)) {
    out << "0";
  } else {
    for (int i=1; i<=team_size; i++)
      if (entries & (1 << (i-1))) {
	out << buf << i;
	buf[0] = ' ';
      }
  }
  out << "}";
  //out << " " << entries;
}

void BallMoveTokenSet::addToken(BallMoveToken t)
{
  int i = (int)t;
  if (i < 1 || i > BallMoveToken_max)
    return;
  if (i==(int)BTO_All) {
    entries = ~0;
  } else {
    entries |= (1 << (i-1));
  }
}

void BallMoveTokenSet::removeToken(BallMoveToken t)
{
  int i = (int)t;
  if (i < 1 || i > BallMoveToken_max)
    return;
  if (i==BTO_All) {
    clear();
  } else {
    entries &= ~(1 << (i-1));
  }
}

bool BallMoveTokenSet::isTokenMember(BallMoveToken t) const
{
  int i = (int)t;
  if (i < 1 || i > BallMoveToken_max)
    return false;
  if (i==BTO_All) {
    return (~(((~0) << ((int)BallMoveToken_max)) | entries)) == 0;
  } else {
    return !!(entries & (1 << (i-1)));
  }
}

void BallMoveTokenSet::Print(ostream& out) const
{
  out << "{";
  for (int i=1; i<=BallMoveToken_max; i++)
    if (entries & (1 << (i-1))) out << BallMoveToken_strings[i];
  out << "}";
  //out << " " << entries;
}


/******** Action primitives ********************/
void CM_ActPosition::iterate(CoachMessageIterator* pIter, IteratorType it) 
{
  if (it == IT_ParentFirst) pIter->iterAction(this);
  pReg->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterAction(this);
}
void CM_ActHome::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterAction(this);
  pReg->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterAction(this);
}
void CM_ActBallToRegion::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterAction(this);
  pReg->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterAction(this);
}
void CM_ActMarkLineRegion::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterAction(this);
  pReg->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterAction(this);
}
void CM_ActOffsidesLine::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterAction(this);
  pReg->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterAction(this);
}
void CM_ActNamed::linkNamedElements(const CoachMessageRepository* pCMR)
{
  pAct = pCMR->lookupAction(name);
}
void CM_ActNamed::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterAction(this);
  if (pAct) pAct->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterAction(this);
}

#ifdef WITH_DRAW
const Magick::Color CM_ActPosition::draw_color       = Magick::ColorRGB(155/255.0, 98/255.0, 70/255.0);
const Magick::Color CM_ActHome::draw_color           = Magick::ColorRGB( 55/255.0,191/255.0, 21/255.0);
const Magick::Color CM_ActBallToRegion::draw_color   = Magick::ColorRGB(214/255.0, 35/255.0,214/255.0);
const Magick::Color CM_ActMarkLineRegion::draw_color = Magick::ColorRGB( 19/255.0,  1/255.0,248/255.0);
const Magick::Color CM_ActOffsidesLine::draw_color   = Magick::ColorRGB(254/255.0,135/255.0, 16/255.0);

void CM_ActPosition::draw(FieldImage* pfi)
{
  pReg->draw(pfi, &draw_color);
}
void CM_ActHome::draw(FieldImage* pfi)
{
  pReg->draw(pfi, &draw_color);
}
void CM_ActBallToRegion::draw(FieldImage* pfi)
{
  pReg->draw(pfi, &draw_color);
  ostrstream out;
  out << "Move ball with: " << bmtset << ends;
  pfi->addLegendLine(out.str());
  out.freeze(0);

}
void CM_ActBallToPlayer::draw(FieldImage* pfi)
{
  ostrstream out;
  out << "Ball to players " << players << ends;
  pfi->addLegendLine(out.str());
  out.freeze(0);
}
void CM_ActMark::draw(FieldImage* pfi)
{
  ostrstream out;
  out << "Mark players " << players << ends;
  pfi->addLegendLine(out.str());
  out.freeze(0);
}
void CM_ActMarkLinePlayer::draw(FieldImage* pfi)
{
  ostrstream out;
  out << "Mark line to players " << players << ends;
  pfi->addLegendLine(out.str());
  out.freeze(0);
}
void CM_ActMarkLineRegion::draw(FieldImage* pfi)
{
  pReg->draw(pfi, &draw_color);
}
void CM_ActOffsidesLine::draw(FieldImage* pfi)
{
  pReg->draw(pfi, &draw_color);
}
void CM_ActHetType::draw(FieldImage* pfi)
{
  char mess[50];
  sprintf(mess, "Het type: %d", player_type);
  pfi->addLegendLine(mess);
}
void CM_ActNamed::draw(FieldImage* pfi)
{
  /*
  char mess[50];
  sprintf(mess, "Action named '%s'", name);
  pfi->addLegendLine(mess);
  */
  if (pAct) pAct->draw(pfi);
  else my_error("ActNamed::draw unlinked");
}
#endif


/******** Directive ********************/
CM_DirCommand::CM_DirCommand( bool positive, bool our_side,
			    const UnumSet& players, CM_Action* pAct)
  : CM_Directive()
{
  this->positive = positive;
  this->our_side = our_side;
  this->players = players;
  this->pAct = pAct;
}


void CM_DirCommand::Print(ostream& out)
{
  out << "("
      << (positive ? "do" : "dont") << " "
      << (our_side ? "our" : "opp") << " "
      << players << " "
      << *pAct << ")";
}

void CM_DirCommand::PrintPretty(ostream& out, string line_header)
{
  out << line_header 
      << (our_side ? "our team" : "opponent") << " "
      << players << " "
      << (positive ? "DO" : "DON'T") << ": " << endl;
  pAct->PrintPretty(out, line_header + " ");
}

void CM_DirCommand::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterDirective(this);
  pAct->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterDirective(this);
}


void CM_DirNamed::linkNamedElements(const CoachMessageRepository* pCMR)
{
  pDir = pCMR->lookupDirective(name);
}

void CM_DirNamed::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterDirective(this);
  if (pDir) pDir->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterDirective(this);
}

#ifdef WITH_DRAW
const Magick::Color CM_DirCommand::draw_color_pos = Magick::ColorRGB(0.0,1.0,0.0);
const Magick::Color CM_DirCommand::draw_color_neg   = Magick::ColorRGB(1.0,0.0,0.0);

void CM_DirCommand::draw(FieldImage* pfi)
{
  ostrstream out;
  out << (our_side ? "Us" : "Them") << " " << players << ends;
  pfi->addLegendLineWithDot(out.str(), positive ? draw_color_pos : draw_color_neg);
  out.freeze(0);
  pAct->draw(pfi);
}

void CM_DirNamed::draw(FieldImage* pfi)
{
  if (pDir) pDir->draw(pfi);
  else my_error("DirNamed::draw unlinked");
}

#endif


/******** Conditions ********************/

/* static member element */
CoachMessageEvaluator* CM_Condition::pEval = NULL;

void CM_CondPlayerPosition::Print(ostream& out)
{
  out << "(ppos"
      << " " << (our_side ? "our" : "opp")
      << " " << players
      << " " << min_match
      << " " << max_match
      << " " << *pReg
      << ")";
}
void CM_CondPlayerPosition::PrintPretty(ostream& out, string line_header)
{
  out << line_header
      << "are "
      << min_match << "-" << max_match << " "
      << (our_side ? "our team" : "opponent") << " "
      << players << " "
      << "in:" << endl;
  pReg->PrintPretty(out, line_header + " ");
}

void CM_CondPlayerPosition::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterCondition(this);
  pReg->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterCondition(this);  
}

void CM_CondBallPosition::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterCondition(this);
  pReg->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterCondition(this);  
}


void CM_CondBallOwner::Print(ostream& out)
{
  out << "(bowner"
      << " " << (our_side ? "our" : "opp") 
      << " " << players
      << ")";  
}
void CM_CondBallOwner::PrintPretty(ostream& out, string line_header)
{
  out << line_header
      << "is ball controlled by someone in " 
      << (our_side ? "our team" : "opponent") << " "
      << players << endl;
}


void CM_CondAnd::Print(ostream& out)
{
  vector<CM_Condition*>::iterator cond_iter;
  out << "(and";
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter) 
    out << " " << **cond_iter;
  out << ")";
}

void CM_CondAnd::PrintPretty(ostream& out, string line_header)
{
  vector<CM_Condition*>::iterator cond_iter;
  out << line_header << "and" << endl;
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter) 
    (*cond_iter)->PrintPretty(out, line_header + " +");
}

void CM_CondAnd::linkNamedElements(const CoachMessageRepository* pCMR)
{
  vector<CM_Condition*>::iterator cond_iter;
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter)
    (*cond_iter)->linkNamedElements(pCMR);
}

void CM_CondAnd::updateNamedCondition(const char* name, CM_Condition* p)
{
  vector<CM_Condition*>::iterator cond_iter;
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter) 
    (*cond_iter)->updateNamedCondition(name, p);
}

void CM_CondAnd::updateNamedRegion   (const char* name, RegionPtr p)
{
  vector<CM_Condition*>::iterator cond_iter;
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter) 
    (*cond_iter)->updateNamedRegion(name, p);
}

void CM_CondAnd::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterCondition(this);
  vector<CM_Condition*>::iterator cond_iter;
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter) 
    (*cond_iter)->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterCondition(this);  
}

CM_Condition* CM_CondAnd::clone()
{
  vector<CM_Condition*> newV;
  vector<CM_Condition*>::iterator cond_iter;
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter) 
    newV.push_back((*cond_iter)->clone());
  return new CM_CondAnd(newV);
}

void CM_CondAnd::clearConditions()
{
  vector<CM_Condition*>::iterator cond_iter;
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter)
    delete (*cond_iter);
  vCond.clear();
}


void CM_CondOr::Print(ostream& out)
{
  vector<CM_Condition*>::iterator cond_iter;
  out << "(or";
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter) 
    out << " " << **cond_iter;
  out << ")";
}

void CM_CondOr::PrintPretty(ostream& out, string line_header)
{
  vector<CM_Condition*>::iterator cond_iter;
  out << line_header << "or" << endl;
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter) 
    (*cond_iter)->PrintPretty(out, line_header + " +");
}

void CM_CondOr::linkNamedElements(const CoachMessageRepository* pCMR)
{
  vector<CM_Condition*>::iterator cond_iter;
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter)
    (*cond_iter)->linkNamedElements(pCMR);
}

void CM_CondOr::updateNamedCondition(const char* name, CM_Condition* p)
{
  vector<CM_Condition*>::iterator cond_iter;
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter) 
    (*cond_iter)->updateNamedCondition(name, p);
}

void CM_CondOr::updateNamedRegion   (const char* name, RegionPtr p)
{
  vector<CM_Condition*>::iterator cond_iter;
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter) 
    (*cond_iter)->updateNamedRegion(name, p);
}

void CM_CondOr::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterCondition(this);
  vector<CM_Condition*>::iterator cond_iter;
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter) 
    (*cond_iter)->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterCondition(this);  
}

CM_Condition* CM_CondOr::clone()
{
  vector<CM_Condition*> newV;
  vector<CM_Condition*>::iterator cond_iter;
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter) 
    newV.push_back((*cond_iter)->clone());
  return new CM_CondOr(newV);
}

void CM_CondOr::clearConditions()
{
  vector<CM_Condition*>::iterator cond_iter;
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter)
    delete (*cond_iter);
  vCond.clear();
}

void CM_CondNot::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterCondition(this);
  pCond->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterCondition(this);  
}

void CM_CondNamed::linkNamedElements(const CoachMessageRepository* pCMR)
{
  pCond = pCMR->lookupCondition(name);
}

void CM_CondNamed::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterCondition(this);
  if (pCond) pCond->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterCondition(this);  
}

#ifdef WITH_DRAW
const Magick::Color CM_CondTrue::draw_color                 = Magick::ColorRGB(0.0,1.0,0.0);
const Magick::Color CM_CondFalse::draw_color                = Magick::ColorRGB(1.0,0.0,0.0);
const Magick::Color CM_CondBallPosition::draw_color         = Magick::ColorRGB(0.7,0.7,0.7);
const Magick::Color CM_CondPlayerPosition::draw_color_left  = Magick::ColorRGB(.75,.75,0.0);
const Magick::Color CM_CondPlayerPosition::draw_color_right = Magick::ColorRGB(1.0,0.0,0.0);

void CM_CondTrue::draw(FieldImage* pfi)
{
  pfi->addLegendLine("TRUE");
}

void CM_CondFalse::draw(FieldImage* pfi)
{
  pfi->addLegendLine("FALSE");
}

void CM_CondPlayerPosition::draw(FieldImage* pfi)
{
  pReg->draw(pfi, our_side ? &draw_color_left : &draw_color_right);
  ostrstream out;
  out << min_match << " - " << max_match << players << ends;
  pfi->addText(out.str(), pReg->getSomePointIn());
  out.freeze(0);
}

void CM_CondBallPosition::draw(FieldImage* pfi)
{
  pReg->draw(pfi, &draw_color);
}

void CM_CondBallOwner::draw(FieldImage* pfi)
{
  ostrstream out;
  out << "Ball owner: " << (our_side ? "Us" : "Them") << " " << players << ends;
  pfi->addLegendLine(out.str());
  out.freeze(0);
}

void CM_CondPlayMode::draw(FieldImage* pfi)
{
  ostrstream out;
  out << "Play mode: " << CM_PlayModeType_strings[pm] << ends;
  pfi->addLegendLine(out.str());
  out.freeze(0);
}

void CM_CondAnd::draw(FieldImage* pfi)
{
  pfi->addLegendLine("AND");
  vector<CM_Condition*>::iterator cond_iter;
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter)
    (*cond_iter)->draw(pfi);
}

void CM_CondOr::draw(FieldImage* pfi)
{
  pfi->addLegendLine("OR");
  vector<CM_Condition*>::iterator cond_iter;
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter)
    (*cond_iter)->draw(pfi);
}

void CM_CondNot::draw(FieldImage* pfi)
{
  pfi->addLegendLine("NOT");
  if (pCond) pCond->draw(pfi);
  else my_error("ContNot::draw NULL");
}

void CM_CondNamed::draw(FieldImage* pfi)
{
  if (pCond) pCond->draw(pfi);
  else my_error("CondNamed::draw unlinked");
}

#endif

/******** Tokens ********************/
void CM_TokRule::Print(ostream& out)
{
  if (!pCond) {
    my_error("Tried to print a TokRule with NULL condition");
    return;
  }
  
  out << "(" << time_to_live << " ";
  out << *pCond;

  vector<CM_Directive*>::iterator dir_iter;
  for (dir_iter = vDir.begin(); dir_iter != vDir.end(); ++dir_iter)
    out << " " << (**dir_iter);
  out << ")";
}
void CM_TokRule::PrintPretty(ostream& out, string line_header)
{
  if (!pCond)
    return;

  out << line_header << "for " << time_to_live << " cycles, if" << endl;
  pCond->PrintPretty(out, line_header + " +");

  out << line_header << "then" << endl;
  vector<CM_Directive*>::iterator dir_iter;
  for (dir_iter = vDir.begin(); dir_iter != vDir.end(); ++dir_iter)
    (*dir_iter)->PrintPretty(out, line_header + " *");
}

void CM_TokRule::clearDirectives()
{
  vector<CM_Directive*>::iterator dir_iter;
  for (dir_iter = vDir.begin(); dir_iter != vDir.end(); ++dir_iter)
    delete (*dir_iter);
  vDir.clear();
}


void CM_TokRule::linkNamedElements(const CoachMessageRepository* pCMR)
{
  pCond->linkNamedElements(pCMR);
  vector<CM_Directive*>::iterator dir_iter;
  for (dir_iter = vDir.begin(); dir_iter != vDir.end(); ++dir_iter)
    (*dir_iter)->linkNamedElements(pCMR);
}

void CM_TokRule::updateNamedCondition(const char* name, CM_Condition* p)
{
  pCond->updateNamedCondition(name, p);
}
void CM_TokRule::updateNamedDirective(const char* name, CM_Directive* p)
{
  vector<CM_Directive*>::iterator dir_iter;
  for (dir_iter = vDir.begin(); dir_iter != vDir.end(); ++dir_iter)
    (*dir_iter)->updateNamedDirective(name, p);
}

void CM_TokRule::updateNamedRegion   (const char* name, RegionPtr p)
{
  pCond->updateNamedRegion(name, p);
  vector<CM_Directive*>::iterator dir_iter;
  for (dir_iter = vDir.begin(); dir_iter != vDir.end(); ++dir_iter) {
    (*dir_iter)->updateNamedRegion(name, p);
  }  
}

void CM_TokRule::updateNamedAction   (const char* name, CM_Action* p)
{
  vector<CM_Directive*>::iterator dir_iter;
  for (dir_iter = vDir.begin(); dir_iter != vDir.end(); ++dir_iter)
    (*dir_iter)->updateNamedAction(name, p);
}

void CM_TokRule::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterToken(this);
  pCond->iterate(pIter, it);
  vector<CM_Directive*>::iterator dir_iter;
  for (dir_iter = vDir.begin(); dir_iter != vDir.end(); ++dir_iter)
    (*dir_iter)->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterToken(this);  
}

CM_Token* CM_TokRule::clone()
{
  vector<CM_Directive*> newV;
  vector<CM_Directive*>::iterator dir_iter;
  for (dir_iter = vDir.begin(); dir_iter != vDir.end(); ++dir_iter)
    newV.push_back((*dir_iter)->clone());
  return new CM_TokRule(time_to_live, pCond->clone(), newV);  
}



#ifdef WITH_DRAW
void CM_TokClear::draw(FieldImage* pfi)
{
  pfi->addLegendLine("CLEAR token");
}

void CM_TokRule::draw(FieldImage* pfi)
{
  pCond->draw(pfi);
  vector<CM_Directive*>::iterator dir_iter;
  for (dir_iter = vDir.begin(); dir_iter != vDir.end(); ++dir_iter)
    (*dir_iter)->draw(pfi);
}

#endif

/******** DefineTokens ********************/

void CM_DefTokCondition::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterDefineToken(this);
  pCond->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterDefineToken(this);  
}

void CM_DefTokDirective::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterDefineToken(this);
  pDir->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterDefineToken(this);  
}

void CM_DefTokRegion::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterDefineToken(this);
  pReg->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterDefineToken(this);  
}

void CM_DefTokAction::iterate(CoachMessageIterator* pIter, IteratorType it)
{
  if (it == IT_ParentFirst) pIter->iterDefineToken(this);
  pAct->iterate(pIter, it);
  if (it == IT_ParentLast) pIter->iterDefineToken(this);  
}

