/* -*- Mode: C++ -*- */

/*
    libscoach: Soccer Coach library for use with the SoccerServer system 

    Copyright (C) 2001  Patrick Riley, Paul Carpenter, Gal Kaminka, Manuela Veloso

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/* The file defines an abstract class to do iteration over the components of
   a coach message */

#ifndef _H_COACH_LANG_ITER
#define _H_COACH_LANG_ITER

#include <stream.h>
#include <iostream.h>

enum IteratorType {
  IT_ParentFirst,
  IT_ParentLast
};

class CM_Message;
class CM_MetaToken;
class CM_Action;
class CM_Directive;
class CM_Condition;
class CM_Token;
class CM_DefineToken;
class Region;

class CoachMessageIterator 
{
public:
  CoachMessageIterator() {}
  virtual ~CoachMessageIterator() {}

  virtual void iterMessage(CM_Message* pMess) = 0;
  virtual void iterMetaToken(CM_MetaToken* pMTok) = 0;
  virtual void iterAction(CM_Action* pAct) = 0;
  virtual void iterDirective(CM_Directive* pDir) = 0;
  virtual void iterCondition(CM_Condition* pCond) = 0;
  virtual void iterToken(CM_Token* pTok) = 0;
  virtual void iterDefineToken(CM_DefineToken* pDTok) = 0;
  virtual void iterRegion(Region* pReg) = 0;
};

#endif
