/* -*- Mode: C++ -*- */

/*
    libscoach: Soccer Coach library for use with the SoccerServer system 

    Copyright (C) 2001  Patrick Riley, Paul Carpenter, Gal Kaminka, Manuela Veloso

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/* This is a root class which the coach and player can inherit from to provide
   and evaluator for the coach messages */

#include "coach_mess_eval.h"
#include "data.h"
#include "LogAction.h"

CMeval_t CoachMessageEvaluator::evalAnd(vector<CM_Condition*> vCond)
{
  MaxMeanSummary results;
  vector<CM_Condition*>::iterator cond_iter;
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter) {
    results.AddPoint((*cond_iter)->eval());
  }
  return results.GetMean();
}

CMeval_t CoachMessageEvaluator::evalOr(vector<CM_Condition*> vCond)
{
  MaxMeanSummary results;
  vector<CM_Condition*>::iterator cond_iter;
  for (cond_iter = vCond.begin(); cond_iter != vCond.end(); ++cond_iter) {
    results.AddPoint((*cond_iter)->eval());
  }
  return results.GetMax();
}

CMeval_t CoachMessageEvaluator::evalNot(CM_Condition* pCond)
{
  float val = pCond->eval();
  LogAction3(210, "Not eval: %.2f", val);
  return 1.0 - val;
}

