/* -*- Mode: C++ -*- */

/*
    libscoach: Soccer Coach library for use with the SoccerServer system 

    Copyright (C) 2001  Patrick Riley, Paul Carpenter, Gal Kaminka, Manuela Veloso

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/* This is a root class which the coach and player can inherit from to provide
   and evaluator for the coach messages */

#include "coach_lang_comp.h"

#ifndef _H_COACH_MESS_EVAL
#define _H_COACH_MESS_EVAL


//class Region;
//class CM_Condition;

class CoachMessageEvaluator 
{
 public:
  CoachMessageEvaluator() {}
  virtual ~CoachMessageEvaluator() {}

  virtual CMeval_t evalPlayerPosition(bool our_side,
				      UnumSet players,
				      int min, int max,
				      RegionPtr pReg) = 0;
  virtual CMeval_t evalBallPosition(RegionPtr pReg) = 0;
  virtual CMeval_t evalBallOwner(bool our_side, UnumSet players) = 0;
  virtual CMeval_t evalPlayMode(CM_PlayModeType pm) = 0;

  /* defaults are provided for these which do:
     and: averages
     or: max
     not: 1-x */
  virtual CMeval_t evalAnd(vector<CM_Condition*> vCond);
  virtual CMeval_t evalOr(vector<CM_Condition*> vCond);
  virtual CMeval_t evalNot(CM_Condition* pCond);

  virtual Vector getBallPos() = 0;
  virtual Vector getPlayerPos(bool our_side, Unum num) = 0;

  bool wasError() { return error_flag; }
  void resetErrorFlag() { error_flag = false; }
  void setErrorFlag() { error_flag = true; }
  
private:
  bool error_flag;
} ;


#endif //_H_COACH_MESS_EVAL
