/* -*- Mode: C++ -*- */

/*
    libscoach: Soccer Coach library for use with the SoccerServer system 

    Copyright (C) 2001  Patrick Riley, Paul Carpenter, Gal Kaminka, Manuela Veloso
    Copyright (C) 1998,1999,2000 Patrick Riley, Peter Stone, Manuela Veloso

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/* Classes dealing with statistical stuff from data points */

#ifndef _DATA_H_
#define _DATA_H_

#include <stdio.h>
#include <fstream.h>
#include <math.h>

//Min and Max are not currently written to or read from a file.
//Maybe some day I'll fix that
class SingleDataSummary {

public:
  SingleDataSummary();

  void Reset();
  
  void AddPoint(double x);
  double GetMean();
  double GetVariance();
  double GetStdDev();
  double GetMin() { return min; }
  double GetMax() { return max; }
  int GetNumPoints();

  void CombineWith(SingleDataSummary* sds);

  void SetPrecision(int prec) { precision = prec; }
  
      
  int  GetPrecision() { return precision; }
  
  void WriteInfoToFile(ostream& output_file, bool WithNumPoints = false);
  void WriteCompactInfoToFile(ostream& output_file, bool WithNumPoints = false);
  void WriteCompactMeanStdevToFile(ostream& output_file, bool WithNumPoints = false);

  /* if num_points is given, it should NOT be in the file */
  bool ReadInfoFromFile(istream& infile, int num_points = -1);
  bool ReadCompactInfoFromFile(istream& infile, int num_points = -1);
  
 private:
  int n;
  double sum;
  double sum_squares;
  double min;
  double max;

  int precision;
};

//Only maintains info for max and mean
class MaxMeanSummary 
{
public:
  MaxMeanSummary() { Reset(); }
  MaxMeanSummary(int new_n, double new_sum, double new_max)
  { n = new_n; sum = new_sum; max = new_max; }

  void Reset();
  
  void AddPoint(double x);
  double GetMean();
  double GetMax() { return max; }
  int GetNumPoints() { return n; }

private:
  int n;
  double sum;
  double max;
} ;

class MinMaxSummary
{
public:
  MinMaxSummary() { Reset(); }

  void Reset() { min = HUGE_VAL; max = -HUGE_VAL; curr_index = 0; mindex = maxdex = -1;}
  
  void AddPoint(double x);
  double GetMin() { return min; }
  double GetMax() { return max; }
  int GetMinIndex() { return mindex; }
  int GetMaxIndex() { return maxdex; }

private:
  double min;
  double max;
  int mindex;
  int maxdex;
  int curr_index;
} ;


#endif
