/* -*- Mode: C++ -*- */

/*
    libscoach: Soccer Coach library for use with the SoccerServer system 

    Copyright (C) 2001  Patrick Riley, Paul Carpenter, Gal Kaminka, Manuela Veloso

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
/* This is a class for a reference counter RegionPtr */

#ifndef _H_REGION_PTR
#define _H_REGION_PTR

class Region; //defined in region.h

class RegionPtr 
{
public:
  RegionPtr() : p(NULL) {}
  explicit RegionPtr(Region* _p);
  RegionPtr(const RegionPtr& rp);
  ~RegionPtr() { decAndCheck(); }

  const RegionPtr& operator=(const RegionPtr& rp);

  Region& operator*() const { return *p; }
  Region* operator->() const { return p; }

  bool operator==(const RegionPtr& rp) const { return p == rp.p; }
  bool operator!=(const RegionPtr& rp) const { return p != rp.p; }
  bool operator==(const void* voidp) const { return p == voidp; }
  bool operator!=(const void* voidp) const { return p != voidp; }

  operator bool () const { return p != NULL; }
  bool operator!() const { return p == NULL; }

  Region* get() const  { return p; }
  int use_count() const;
  bool unique() const;

  Region* forget();
  void setNULL() { decAndCheck(); p = NULL; }
  
private:
  void decAndCheck();
  Region* p;
};

#include "region.h"

inline RegionPtr::RegionPtr(Region* _p) : p(_p) { if (p) p->reference_count++; }
inline RegionPtr::RegionPtr(const RegionPtr& rp) { p = rp.p; if (p) p->reference_count++; }

inline const RegionPtr& RegionPtr::operator=(const RegionPtr& rp)
{ decAndCheck(); p = rp.p; if (p) p->reference_count++; return *this; }

inline int RegionPtr::use_count() const { return (p == NULL) ? -1 : p->reference_count; }  
inline bool RegionPtr::unique() const  { return (p==NULL) ? true : (p->reference_count == 1); }

inline void RegionPtr::decAndCheck() { if (p==NULL) return; if (--p->reference_count == 0) delete p; }

inline Region* RegionPtr::forget()
{ Region* tmp = p; if (p) --p->reference_count; p = NULL; return tmp; }
 

#endif
