/* -*- Mode: C++ -*- */

/*
    libscoach: Soccer Coach library for use with the SoccerServer system 

    Copyright (C) 2001  Patrick Riley, Paul Carpenter, Gal Kaminka, Manuela Veloso
    Copyright (C) 1998,1999,2000 Patrick Riley, Peter Stone, Manuela Veloso

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef _TYPES_ALL_H_
#define _TYPES_ALL_H_

typedef bool Bool;
#define FALSE ((bool)false)
#define TRUE ((bool)true)

enum FuzzyBool {
  FB_False,
  FB_True,
  FB_Unknown,
  FB_Maybe,
  FM_NoValue
};

typedef float Value;
typedef float AngleRad;
typedef float AngleDeg;

#define NULLCHAR	'\000'

typedef int Unum;  /* Uniform number           */
#define Unum_Unknown 0

const int num_players  = 11;
const int bits_for_player_num = 4;

struct option_t {
   char	optname[50] ;
   void	*vptr ;
   int	vsize ;
};

enum InputType{
 V_INT,
 V_FLOAT,
 V_BOOL,
 V_STRING,	
 V_ONOFF,		
 V_NONE		
};

#define MAX_TEAMNAME_LEN  20
#define MAX_HOST_LEN      50
#define MAX_FILE_LEN      256
#define MAX_FP_LEN        20
#define MAX_TREE_STEM_LEN 50
#define MAX_ADVCLASS_NAME_LEN 50

const int MAX_MESSAGE_CHARS = 128;

//These must start and 0 and be continuous
const int num_SetPlayMode = 13;
enum SetPlayMode{
 SPM_No_Mode,
 SPM_My_Kick_Off,
 SPM_Their_Kick_Off,
 SPM_My_Kick_In,
 SPM_Their_Kick_In,
 SPM_My_Corner_Kick,
 SPM_Their_Corner_Kick,
 SPM_My_Goal_Kick,
 SPM_Their_Goal_Kick,
 SPM_My_Free_Kick,
 SPM_Their_Free_Kick,
 SPM_My_Goalie_Catch,
 SPM_Their_Goalie_Catch
};

#define SETPLAYMODE_STRINGS {\
"NoMode",\
"MyKickOff",\
"TheirKickOff",\
"MyKickIn",\
"TheirKickIn",\
"MyCornerKick",\
"TheirCornerKick",\
"MyGoalKick",\
"TheirGoalKick",\
"MyFreeKick",\
"TheirFreeKick",\
"MyGoalieCatch",\
"TheirGoalieCatch" }

//These must start and 0 and be continuous
//These are slightly more specialized modes from the ones above
//Right now, the coach uses these to get good seeds for hillclimbing
const int num_SpecSetPlayMode = 21;
enum SpecSetPlayMode{
 SSPM_No_Mode,
 SSPM_My_Kick_Off,
 SSPM_Their_Kick_Off,
 SSPM_My_Kick_In_Def, //defensive for us
 SSPM_My_Kick_In_Mid,
 SSPM_My_Kick_In_Off,
 SSPM_Their_Kick_In_Def, //defensive for them
 SSPM_Their_Kick_In_Mid,
 SSPM_Their_Kick_In_Off,
 SSPM_My_Corner_Kick,
 SSPM_Their_Corner_Kick,
 SSPM_My_Goal_Kick,
 SSPM_Their_Goal_Kick,
 //we assume free kicks are defensive since the only we really get them are offsides
 SSPM_My_Free_Kick_Upper,
 SSPM_My_Free_Kick_Mid,
 SSPM_My_Free_Kick_Lower,
 SSPM_Their_Free_Kick_Upper,
 SSPM_Their_Free_Kick_Mid,
 SSPM_Their_Free_Kick_Lower,
 SSPM_My_Goalie_Catch,
 SSPM_Their_Goalie_Catch
};

#define SPECSETPLAYMODE_STRINGS {\
 "NoMode",\
 "MyKickOff",\
 "TheirKickOff",\
 "MyKickInDef",\
 "MyKickInMid",\
 "MyKickInOff",\
 "TheirKickInDef",\
 "TheirKickInMid",\
 "TheirKickInOff",\
 "MyCornerKick",\
 "TheirCornerKick",\
 "MyGoalKick",\
 "TheirGoalKick",\
 "MyFreeKickUpper",\
 "MyFreeKickMid",\
 "MyFreeKickLower",\
 "TheirFreeKickUpper",\
 "TheirFreeKickMid",\
 "TheirFreeKickLower",\
 "MyGoalieCatch",\
 "TheirGoalieCatch" }


typedef enum Ptype{
  PT_None,
  PT_Goaltender,
  PT_Sweeper,
  PT_Defender,
  PT_Midfielder,
  PT_Forward
} PlayerRole;


/* Coach language stuff */

typedef double CMeval_t;
const CMeval_t CME_true = 1.0;
const CMeval_t CME_false = 0.0;


//It is important that BTO_None is first and BTO_All is last
enum BallMoveToken
{ BTO_None,
  BTO_Pass,
  BTO_Dribble,
  BTO_Clear,
  BTO_Score,
  BTO_All };
extern char* BallMoveToken_strings[];
const int BallMoveToken_max = 4;

enum CM_PlayModeType
{
  CMPM_None,
  CMPM_BeforeKickOff,
  CMPM_TimeOver,
  CMPM_PlayOn,
  CMPM_KickOff_Our,
  CMPM_KickOff_Opp,
  CMPM_KickIn_Our,
  CMPM_KickIn_Opp,
  CMPM_FreeKick_Our,
  CMPM_FreeKick_Opp,
  CMPM_CornerKick_Our,
  CMPM_CornerKick_Opp,
  CMPM_GoalKick_Our,
  CMPM_GoalKick_Opp,
  CMPM_GoalieCatch_Our,
  CMPM_GoalieCatch_Opp,
  CMPM_AfterGoal_Our,
  CMPM_AfterGoal_Opp
};
extern char* CM_PlayModeType_strings[];

enum CM_MetaTokenType {
  CMMT_Version
};

enum CM_ActionType {
  CMA_Position,
  CMA_Home,
  CMA_BallToRegion,
  CMA_BallToPlayer,
  CMA_Mark,
  CMA_MarkLinePlayer,
  CMA_MarkLineRegion,
  CMA_OffsidesLine,
  CMA_HetType,
  CMA_Named
};

enum CM_DirectiveType {
  CMD_Command,
  CMD_Named,
};

enum CM_ConditionType {
  CMC_True,
  CMC_False,
  CMC_PlayerPosition,
  CMC_BallPosition,
  CMC_BallOwner,
  CMC_PlayMode,
  CMC_And,
  CMC_Or,
  CMC_Not,
  CMC_Named
};

enum CM_TokenType {
  CMT_Clear,
  CMT_Rule
};

enum CM_DefineTokenType {
  CMDT_Condition,
  CMDT_Directive,
  CMDT_Region,
  CMDT_Action
};

#endif
