/* -*- Mode: C++ -*- */

/*
    libscoach: Soccer Coach library for use with the SoccerServer system 

    Copyright (C) 2001  Patrick Riley, Paul Carpenter, Gal Kaminka, Manuela Veloso
    Copyright (C) 1998,1999,2000 Patrick Riley, Peter Stone, Manuela Veloso

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef _UTILS_
#define _UTILS_

#include <math.h>
#include <fstream.h>
#include "types_all.h"
#include <string.h> /* iet: 4/30/01 */

void dump_core(char*);

/* This my_error needs to be defined my whoever uses this */
extern void my_error(const char*);

void my_error(const char*,int);
void my_error(const char*,int,int);
void my_error(const char*,int,int,int);
void my_error(const char*,int,int,int,int);
void my_error(const char*,int,int,int,int,int);
void my_error(const char*,int,int,int,int,int,int);
void my_error(const char*,int,int,int,int,int,char,int);
void my_error(const char*,float);
void my_error(const char*,float,float);
void my_error(const char*,float,float,float);
void my_error(const char*,float,float,float,float);
void my_error(const char*,float,float,float,float,float);
void my_error(const char*,float,float,float,float,float,float);
void my_error(const char*,float,float,float,float,float,float,float);
void my_error(const char*,float,int);
void my_error(const char*,int,float);
void my_error(const char*,int,int,int,float);
void my_error(const char*,const char*);
void my_error(const char*,char,int,int);
void my_error(const char*,char,int,float,float);
void my_error(const char*, const char*, int);
void my_error(const char*, const char*, int, int);
void my_error(const char*, const char*, const char*);
void my_error(const char*, const char*, const char*, const char*);


int closest_int( float x );

inline AngleDeg Rad2Deg(AngleRad x) { return x * 180 / M_PI; }
inline AngleRad Deg2Rad(AngleDeg x) { return x * M_PI / 180; }

/* needed? */
/* inline float cos(AngleRad x) { return cos((float) x); } */
/* inline float sin(AngleRad x) { return sin((float) x); } */
/* inline float tan(AngleRad x) { return tan((float) x); } */

inline float Cos(AngleDeg x) { return cos(Deg2Rad(x)); }
inline float Sin(AngleDeg x) { return sin(Deg2Rad(x)); }
inline float Tan(AngleDeg x) { return tan(Deg2Rad(x)); }
inline AngleDeg ACos(float x)           { return ((x) >= 1 ? 0 : ((x) <= -1 ? 180 : (Rad2Deg(acos(x))))); }
inline AngleDeg ASin(float x)           { return ((x) >= 1 ? 90 : ((x) <= -1 ? -90 : (Rad2Deg(asin(x))))); }
inline AngleDeg ATan(float x)           { return (Rad2Deg(atan(x))); }
inline AngleDeg ATan2(float x, float y) { return ((x == 0 && y == 0) ? 0 : (Rad2Deg(atan2(x,y)))); } 

void NormalizeAngleDeg(int*);
void NormalizeAngleDeg(AngleDeg*);
void NormalizeAngleDeg(double*);
void NormalizeAngleRad(AngleRad*);
AngleDeg GetNormalizeAngleDeg(AngleDeg);
double GetNormalizeAngleRad(double);

float GetDistance(float *x, float *y, float *a, float *b);

const double FLOAT_EPS = .00001;

#define Mod(a,b) (a - (b)*(int)((a)/(b)))
#define Sign(x) ((x) >= 0 ? 1 : -1)
#define Min(x,y) ((x) < (y) ? (x) : (y))
#define Max(x,y) ((x) > (y) ? (x) : (y))
#define MinMax(min, x, max) Min(Max((min),(x)), (max))

//From Dave M.
#define sigmoid(x) (1/(1+exp(-x)))
#define sigmoid_deriv(x) (1/((1+exp(-x))*(1+exp(x))))
#define soft_not(x) (1-(x))
#define soft_and(x, y) ((x)*(y))
#define soft_or(x,y) (1-(1-(x))*(1-(y)))
#define soft_greater(x,y,delta) (sigmoid(((x)-(y))/(delta)))
#define soft_less(x,y,delta) (sigmoid(((y)-(x))/(delta)))
inline float soft_equal(float x, float y, float delta){
  float z = (x-y)/delta;
  return exp(-z*z);
}

inline float soft_if(float p, float x, float y){
  return p*x + (1-p)*y;
}


float int_pow(float x, int p);
inline int Sqr(int x){ return x*x; }
inline float Sqr(float x) { return x*x; }
inline double Sqr(double x) { return x*x; }
inline  float Exp(float x, int y) { float a = 1; for (int i=0; i<y; i++) a*=x; return a; }

inline float SumInfGeomSeries(float first_term, float r)
{ return first_term / (1 - r); }
float SumGeomSeries(float first_term, float r, int n);
/* returns -1 on error */
float LengthGeomSeries(float first_term, float r, float last_term);
inline float SumGeomSeriesGivenLast(float first_term, float r, float last_term)
{ return SumGeomSeries(first_term, r, 
		       closest_int(LengthGeomSeries(first_term, r, last_term))); 
}
float SolveForLengthGeomSeries(float first_term, float r, float sum);
float SolveForFirstTermGeomSeries(float r, int n, float sum);
inline float SolveForFirstTermInfGeomSeries(float r, float sum)
{ return sum * (1 - r); }

inline double gaussian_func(double x, double mean = 0.0, double stdev = 1.0)
{
  static const double piterm = 1.0 / sqrt(2 * M_PI);

  return (1.0 / stdev) * piterm * pow(M_E, - Sqr(x-mean) / (2*Sqr(stdev)));
}
double gaussian_sample(double mean = 0.0, double stdev = 1.0);
double gaussian_sample_twosided(double mean, double stdev_minus, double stdev_plus);

#define signf(x) ( ((x) >= 0.0) ? 1.0 : -1.0) 
float Round(float x, int p=0);
inline int RoundToInt(float x) 
{
  return (int)rint((double)x);
} 

inline float WeightedSum (float v1, float w1, float v2)
{
  return (w1*v1 + (1.0-w1)*v2);
}
inline float WeightedSum (float v1, float w1, float v2, float w2)
{
  return (w1*v1 + w2*v2) / (w1+w2);
}


extern const char char_for_num_array[16];
inline char char_for_num(int num)
{ return char_for_num_array[num]; }
int num_for_char(char c);


/* returns a pointer to a static buffer, so be careful! */
char* repeat_char(char c, int n);


Bool skip_line(istream& infile);
// returns FALSE if eof
Bool skip_white_space(istream& infile) ;
// returns FALSE if eof
Bool skip_to_non_comment(istream& infile, char comm_char = '#');
/* advances to the specifed character, but only on the same line */
Bool advance_to_on_line(istream& infile, char c);
/* path goes in, dir goes out */
/* returns a pathname with a trailing / */
void get_leading_directory(const char* path, char* dir); 

/* puts the number after the colon in the int, returns FALSE on failure */
Bool GetIntFromLine(char* line, int* pval); 
/* puts the number after the colon in the float, returns FALSE on failure */
Bool GetFloatFromLine(char* line, float* pval); 
Bool GetDoubleFromLine(char* line, double* pval); 
/* puts the number after the colon in the int, returns FALSE on failure */
Bool GetIntOffLine(char** pline, int* pval); 
/* puts the number after the colon in the float, returns FALSE on failure */
Bool GetDoubleOffLine(char** pline, double* pval); 


extern double get_double(char **str_ptr);
extern double get_double(char *str);
extern float  get_float (char **str_ptr);
extern float  get_float (char *str);
extern int    get_int   (char **str_ptr);
extern int    get_int   (char *str);
extern void   get_word  (char **str_ptr);
extern void   get_next_word (char **str_ptr);
extern void   get_token  (char **str_ptr);
extern void   advance_to(char c, char **str_ptr);
extern void   advance_past_space(char **str_ptr);

extern int put_int(char *str, int num);
extern int put_float(char *str, float fnum, int precision);

extern void BubbleSort  (int length, int *elements, float *keys);
extern int  BinarySearch(int length, float *elements, float key);
extern void StrReplace  (char *str, char oldchar, char newchar);

extern int   int_random(int n);
extern float range_random(float lo, float hi);
extern int   very_random_int(int n);

extern float weighted_avg(float val1, float val2, float w1, float w2);

extern void GetStampedName( char *name, char *outputName );

//you give it a series of void* pointers and score and then you can get out the best N
class RecordBestN 
{
public:
  RecordBestN(int N);
  ~RecordBestN();
  
  void Clear();
  
  void AddPoint (void* data, float score);
  //0th best is the best
  void* GetBest(int idx = 0) { return (idx < num_stored) ? arrData[idx] : NULL; }
  float GetBestScore(int idx = 0) { return (idx < num_stored) ? arrScore[idx] : 0.0; }
  int GetNumStored() { return num_stored; }
  
private:
  int N;
  int num_stored;
  void** arrData;
  float* arrScore;
  
} ;

/* Random useful STL stuff */
struct ltstr
{
  bool operator()(const char* s1, const char* s2) const
  {
    return strcmp(s1, s2) < 0;
  }
};

struct ltstrnocase
{
  bool operator()(const char* s1, const char* s2) const
  {
    return strcasecmp(s1, s2) < 0;
  }
};



#endif
